C ************************************************************************
C ***              PROGRAM     'vdW-WF.f'
C ************************************************************************
C
C *** COMPUTES THE VdW CORRECTION BY USING THE POSITIONS OF THE
C *** WANNIER FUNCTION CENTERS AND THE WANNIER FUNCTION SPREADS
C *** BY USING THE OUTPUT FROM THE 'WanT' CODE 
C *** (http://www.wannier-transport.org)
C *** DFT/vdW-WF method
C *** (PRL 100, 053002 (2008), JPC A 113 (2009) 5224);
C *** or
C *** DFT/vdW-WF2 method
C *** (Phys. Rev. B 85 (2012) 073101)
C ***
C *** THE INPUT FILE IS READ FROM THE STANDARD INPUT (UNIT 5)
C *** AND HAS THE FOLLOWING STRUCTURE:
C *** 
C *** iswitchvdw
C *** idefinition
C *** zlevel (IF idefinition=1)
C *** nfrag (IF idefinition=2)
C *** idum,ifragion(i),radfrag(i) (i=1,nfrag) (IF idefinition=2)
C *** nwfc,nelpwf,izcenter
C *** a6,efrag,screen
C *** a1(1)   a1(2)    a1(3)
C *** a2(1)   a2(2)    a2(3)
C *** a3(1)   a3(2)    a3(3)
C *** nrep 
C *** R(a.u.)   eks(Ry)      
C *** WANNIER-file 
C *** R(a.u.)   eks(Ry)      
C *** WANNIER-file 
C ***   .         .               
C ***   .         .              
C ***   .         .             
C *** R(a.u.)   eks(Ry)      
C *** WANNIER-file 
C ***
C ***
C *** INPUT PARAMETER EXPLANATION :
C ***
C *** iswitchvdw  = selected method (1=DFT/vdW-WF, 2=DFT/vdW-WF2,
C ***               3=DFT/vdW-WF2 with overlap computed for the 
C ***                 first configuration only);
C *** idefinition = criterion to assign Wannier function centers (WFCs)
C ***               to different fragments:
C ***               1= 2 fragments separated by zlevel (along z)
C ***                  (suitable to surface adsorption)
C ***               2= multiple fragments; to each fragment the
C ***                  WFCs enclosed within a
C ***                  given radius around a given ion are assigned;
C *** zlevel(a.u.)= IF idefinition=1 WFCs below (above) zlevel (in a.u.) 
C ***                  are assigned to first (second) fragment;
C *** nfrag       = IF idefinition=2 number of fragments;
C *** ifragion(i) = IF idefinition=2 number of the ion taken as the center
C ***                  of the i-th fragment;
C *** radfrag(i)  = IF idefinition=2 radius (in a.u.) around the ion taken 
C ***                  as the center of the i-th fragment;
C *** nwfc        = total number of WFCs;
C *** nelpwf      = number of electrons per Wannier function (usually 2
C ***               if spin degeneracy applies); 
C *** izcenter    = IF = 1 WFCs are moved between -z/2 and +z/2 ;
C *** a6          = adimensional parameter for the damping function:
C ***               f(r)=1/[1+exp(-a6*(r/Rs-1))], SUGGESTED VALUE=20
C ***               (see Wu and Yang, JCP 116 (2002) 515;
C ***                Grimme, J. Comput. Chem. 25 (2004) 1463); 
C *** efrag(Ry)   = sum of the total energy (in Ry) of all the 
C ***               ISOLATED fragments (without vdW corrections) taken
C ***               as a reference value to compute the binding energy;
C *** screen      = C6 coefficients are multiplied by (1-screen), which
C ***               represents a suitable adimensional screening 
C ***               parameter (for instance for adsorption on metal
C ***               surfaces the C6 coefficients could be reduced to
C ***               reproduce the asymptotic C3 coefficient obtained by
C ***               a jellium model); if screen=0.0 no screening effect;  
C *** ai(j)(a.u.) = j-th component (in a.u.) of the i-th supercell
C ***               lattice vector;
C *** nrep        = if nrep > 0 4*nrep*(nrep+1) periodic replicas of 
C ***               the original supercell are considered in the (x,y) plane
C ***               (usually adopted in surface adsorption calculations);
C *** R(a.u.)     = IF nfrag=2 is the interfragment distance (in a.u.),
C ***               otherwise can be considered a dummy variable;
C *** eks(Ry)     = Kohn-Sham total energy (in Ry) without vdW corrections;
C *** WANNIER-file= name of the (WanT) ouput file containing Wannier
C ***               function data.
C ***
C ***
C *** THE OUTPUT FILE WRITTEN IN THE STANDARD OUTPUT (UNIT 6)
C *** HAS THE FOLLOWING STRUCTURE :
C *** 
C ***    R(a.u.)  EKS(Ry)     EVDW(Ry)   averaged C6(a.u.)
C ***    R(a.u.)  EKS(Ry)     EVDW(Ry)   averaged C6(a.u.)
C ***    .       .           .          .      . 
C ***    .       .           .          .      .  
C ***    .       .           .          .      .   
C *** 
C ***
C *** WHERE EVDW = vdW correction energy. 
C ***
C *** THE ADDITIONAL OUTPUT FILE (UNIT 14) 'Ebind_meV.dat'  CONTAINS:
C ***
C *** R(a.u.) R(A)  Ebind(meV)    Ebind+EVDW(meV)    EVDW(meV)
C *** R(a.u.) R(A)  Ebind(meV)    Ebind+EVDW(meV)    EVDW(meV)
C ***    .       .      .               .                .        
C ***    .       .      .               .                .        
C ***    .       .      .               .                .        
C ***
C *** WHERE Ebind  = eks - efrag
C ***
C ************************************************************************
C
        implicit real*8 (a-h,o-z)
        include 'max-dimension.h'
        character*30 filein
c
        dimension ifragion(ntotfragx),fove(ntotfragx)
        dimension radfrag(ntotfragx),icontfragw(ntotfragx)
        dimension c6all(nwfcx,nwfcx,ntotfragx,ntotfragx)
        dimension a1(3),a2(3),a3(3),h(3,3),hm1(3,3)
        dimension rwfctmp(3,nwfcx,ntotfragx),sprtmp(nwfcx,ntotfragx)
        dimension indx(nwfcx),zsort(nwfcx)
c
        write(6,*)
        write(6,*) ' VAN DER WAALS CORRECTION USING WANNIER FUNCTIONS'
        write(6,*) ' ( WANNIER FUNCTION CENTERS AND SPREADS)'
        write(6,*)
c
c numerical constants :
c
        ev=2.*13.6058
        evm=1000.*ev
        au=0.529177
        emin=1000.
        emin6=1000.
        ieffvol=0
c
        read(5,*) iswitchvdw
        if (iswitchvdw.eq.1) then
         write(6,*)' DFT/vdW-WF method'
         write(6,*)'  PRL 100, 053002 (2008), JPC A 113 (2009) 5224'
        else if (iswitchvdw.eq.2.or.iswitchvdw.eq.3) then
         write(6,*)' DFT/vdW-WF2 method'
         write(6,*)'  Phys. Rev. B 85 (2012) 073101'
         write(6,*) ' SIMPLE London formulation'
         write(6,*) ' with reference H polarizability and vdW radius,'
         write(6,*) ' INTRAFRAGMENT MULTIPLE OVERLAP is considered.'
         if (iswitchvdw.eq.3) then
          write(6,*) ' N.B. OVERLAP IS COMPUTED EXPLICITLY'
          write(6,*) '      FOR THE FIRST CONFIGURATION ONLY.'
         endif
         write(6,*)
        else
         stop ' wrong iswitchvdw !'
        endif
c
        read(5,*) idefinition
        if (idefinition.eq.1) then
         read(5,*) zlevel
         write(6,*)
         write(6,*) ' 2 fragments defined by z-coord. level(a.u.) =',
     . zlevel
         write(6,*)
         nfrag=2
        else if (idefinition.eq.2) then
         write(6,*)
         write(6,*) ' multiple fragments defined by radius'
         write(6,*)
         read(5,*) nfrag
         if (nfrag.gt.ntotfragx) then
          write(6,*)
          write(6,*) ' ntotfragx too small ! '
          write(6,*) ' increase ntotfragx in max-dimension.h'
          stop
         endif
         do i=1,nfrag
          read(5,*) idum,ifragion(i),radfrag(i)
         enddo
        else
         stop ' wrong idefinition !'
        endif
        read(5,*) nwfc,nelpwf,izcenter
        facspin=sqrt(float(nelpwf))
        if (nwfc.gt.nwfcx) then 
         write(6,*) 
         write(6,*) ' nwfcx too small ! '
         write(6,*) ' increase nwfcx in max-dimension.h'
         stop 
        endif
        if (izcenter.eq.1) then
         write(6,*)
         write(6,*) ' Wannier function centers are moved'
         write(6,*) ' between -z/2 and + z/2 !!!'
         write(6,*)
        endif
        read(5,*) a6,efrag,screen
c
c from Ry to a.u.:
c
        efrag=efrag/2.
c
        if (screen.ne.0.0) write(6,302) screen
302     format(1x,'SCREENING FACTOR (from input):',f6.3/)
        if (a6.lt.1.0D-05) then
         write(6,*) 
         write(6,*) ' Damping factor too small. Set to 20.0',
     .                   ' according to:'
         write(6,*)'            Mol. Phys. 107, 999 (2009)'
         a6=20.0D0
        endif
c
        read(5,*) (a1(j),j=1,3)
        read(5,*) (a2(j),j=1,3)
        read(5,*) (a3(j),j=1,3)
c
        read(5,*) nrep
        if (nrep.ne.0.) then
         write(6,*)
         write(6,*) 4*nrep*(nrep+1),' XY supercell REPLICAS !!!' 
         write(6,*)
         if (idefinition.ne.1) then
          write(6,*)' WARNING: possible problems with REPLICAS !!!'
         endif
        endif
c
c  creates the lattice vector matrix:
c
        do j=1,3
         h(1,j)=a1(j)
         hm1(1,j)=h(1,j)
         h(2,j)=a2(j)
         hm1(2,j)=h(2,j)
         h(3,j)=a3(j)
         hm1(3,j)=h(3,j)
        enddo
c
        cosen=0.
        a1n=0.
        a2n=0.
        do ic=1,3
         cosen=cosen+a1(ic)*a2(ic)
         a1n=a1n+a1(ic)*a1(ic)
         a2n=a2n+a2(ic)*a2(ic)
        enddo
        cosen=cosen/dsqrt(a1n*a2n)
        symtest=dabs(dabs(cosen)-0.5)
        if (symtest.lt.1.e-3) then
         write(6,*) '        HEXAGONAL PBCs '
         write(6,*)
        else
         write(6,*) '        GENERAL PBCs '
         write(6,*)
        endif
c
c inverts the lattice vector matrix :
c
        CALL gaussj(hm1,3)
c
        open(unit=14,file='Ebind_meV.dat',status='unknown')
c
999     continue
c
c Main loop on a given set of WFCs:
c
        read(5,*,end=1000) r,eks
        read(5,727) filein
727     format(a30)
c
c from Ry to a.u.:
c
        eks=eks/2.
c
        c6ave=0.
        icont=0
        distave=0.
        distmin=1000.
        sijave=0.
        EVDW=0.
c
        open(unit=10,file=filein,status='unknown')
        call readwfc(izcenter)
        close(10)
        write(6,*)
c
c  identify the WFCs belonging to each fragment:
c
        if (idefinition.eq.1) then      ! adopts the z-level criterion
         icontfragw(1)=0
         icontfragw(2)=0
         do jcfrag=1,nwfc
          zlm=rwfc(3,jcfrag,1)
          if(zlm.lt.zlevel) then
           icontfragw(1)=icontfragw(1)+1
           do ic=1,3
            rwfctmp(ic,icontfragw(1),1)=rwfc(ic,jcfrag,1)
           enddo
           sprtmp(icontfragw(1),1)=spr(jcfrag,1)
          else
           icontfragw(2)=icontfragw(2)+1
           do ic=1,3
            rwfctmp(ic,icontfragw(2),2)=rwfc(ic,jcfrag,1)
           enddo
           sprtmp(icontfragw(2),2)=spr(jcfrag,1)
          endif
         enddo
         write(6,3330) 1,icontfragw(1)
         write(6,3330) 2,icontfragw(2)
3330     format(1x,' FRAGMENT',i8,'  CONTAINS',i10,'  WFCs')
         do icfrag=1,2
          do jcfrag=1,icontfragw(icfrag)
           rwfc(1,jcfrag,icfrag)=rwfctmp(1,jcfrag,icfrag)
           rwfc(2,jcfrag,icfrag)=rwfctmp(2,jcfrag,icfrag)
           rwfc(3,jcfrag,icfrag)=rwfctmp(3,jcfrag,icfrag)
           spr(jcfrag,icfrag)=sprtmp(jcfrag,icfrag)
          enddo
         enddo
c
        else if (idefinition.eq.2) then ! adopts the fragment radius criterion
c ionic coordinates are read : 
c
         open(unit=11,file=filein,status='unknown')
         call readion
         close(11)
c
         do icfrag=1,nfrag
          icontfragw(icfrag)=0
          ii=ifragion(icfrag)
c
          do jcfrag=1,nwfc
           dis1=rwfc(1,jcfrag,1)-rion(1,ii)
           dis2=rwfc(2,jcfrag,1)-rion(2,ii)
           dis3=rwfc(3,jcfrag,1)-rion(3,ii)
c
           if (symtest.lt.1.e-3) then
            call hexpbc(dis1,dis2,dis3,xdis1,xdis2,xdis3,a1,a2,a3)
           else
            call mypbc(dis1,dis2,dis3,xdis1,xdis2,xdis3,h,hm1)
           endif
           distfrag=dsqrt(xdis1*xdis1+xdis2*xdis2+xdis3*xdis3)
           if(distfrag.lt.radfrag(icfrag)) then
            icontfragw(icfrag)=icontfragw(icfrag)+1
            rwfctmp(1,icontfragw(icfrag),icfrag)=rwfc(1,jcfrag,1)
            rwfctmp(2,icontfragw(icfrag),icfrag)=rwfc(2,jcfrag,1)
            rwfctmp(3,icontfragw(icfrag),icfrag)=rwfc(3,jcfrag,1)
            sprtmp(icontfragw(icfrag),icfrag)=spr(jcfrag,1)
           endif
          enddo
          write(6,3330) icfrag,icontfragw(icfrag)
c
         enddo
c
         do icfrag=1,nfrag
          do jcfrag=1,icontfragw(icfrag)
           rwfc(1,jcfrag,icfrag)=rwfctmp(1,jcfrag,icfrag)
           rwfc(2,jcfrag,icfrag)=rwfctmp(2,jcfrag,icfrag)
           rwfc(3,jcfrag,icfrag)=rwfctmp(3,jcfrag,icfrag)
           spr(jcfrag,icfrag)=sprtmp(jcfrag,icfrag)
          enddo
         enddo
c
        endif
c
        if (iswitchvdw.eq.2.or.iswitchvdw.eq.3) then
         if (ieffvol.eq.1) goto 777
c
c computes effective volume of each fragment :
c
         do icfrag=1,nfrag
c
          call effvol(icfrag,icontfragw(icfrag),effv)
          fove(icfrag)=(effv)**(1./3.)
c
         enddo
         if (iswitchvdw.eq.3) ieffvol=1
c
        endif
777     continue
c  Computes C6 coefficients
c  Exact H polarizability is enforced:
c
c        c6fac=0.5D0*facspin/(3.D0**1.25D0)
        c6fac=facspin/(3.D0**1.25D0)
c
        do icfrag=1,nfrag ! scan all fragments
         do jcfrag=1,nfrag
c
          if(jcfrag.ne.icfrag) then
           do l=1,icontfragw(icfrag)
            do m=1,icontfragw(jcfrag)
c
             if (iswitchvdw.eq.2.or.iswitchvdw.eq.3) then  ! vdw-WF2
              seffi=spr(l,icfrag)*fove(icfrag)
              seffj=spr(m,jcfrag)*fove(jcfrag)
              seffi3=seffi*seffi*seffi
              seffj3=seffj*seffj*seffj
              seffi32=dsqrt(seffi3)
              seffj32=dsqrt(seffj3)
c              C6=1.209D0*dsqrt(2.D0)*seffi3*seffj3/(seffj32+seffi32)
              C6=1.209D0*facspin*seffi3*seffj3/(seffj32+seffi32)
              c6all(l,m,icfrag,jcfrag)=C6
c
             else if (iswitchvdw.eq.1) then  ! vdw-WF
c
              si3=spr(l,icfrag)*spr(l,icfrag)*spr(l,icfrag)
              sj3=spr(m,jcfrag)*spr(m,jcfrag)*spr(m,jcfrag)
              si3d2=dsqrt(si3)
              call DOUBLES(spr(l,icfrag),spr(m,jcfrag),RDOUBLES)
              c6all(l,m,icfrag,jcfrag)=si3d2*sj3*c6fac*RDOUBLES
c
             endif
            enddo
           enddo
          endif
c
         enddo
        enddo
c
        do icfrag=1,nfrag
         do jcfrag=icfrag+1,nfrag
          do l=1,icontfragw(icfrag)
           do m=1,icontfragw(jcfrag)
c
            dis1=rwfc(1,m,jcfrag)-rwfc(1,l,icfrag)
            dis2=rwfc(2,m,jcfrag)-rwfc(2,l,icfrag)
            dis3=rwfc(3,m,jcfrag)-rwfc(3,l,icfrag)
            if (symtest.lt.1.e-3) then
             call hexpbc(dis1,dis2,dis3,xdis1,xdis2,xdis3,a1,a2,a3)
            else
             call mypbc(dis1,dis2,dis3,xdis1,xdis2,xdis3,h,hm1)
            endif
creplicas
            do ir1=-nrep,nrep
             do ir2=-nrep,nrep
              icont=icont+1
              xxdis1=xdis1+float(ir1)*a1(1)+float(ir2)*a2(1)
              xxdis2=xdis2+float(ir1)*a1(2)+float(ir2)*a2(2)
c
              distn=sqrt(xxdis1*xxdis1+xxdis2*xxdis2+xdis3*xdis3)
              distave=distave+distn
              if (distn.lt.distmin) distmin=distn
              distn6=distn*distn*distn*distn*distn*distn
              c6vdw=c6all(l,m,icfrag,jcfrag)*(1.-screen)
              c6ave=c6ave+c6vdw
              sijave=sijave+spr(l,icfrag)+spr(m,jcfrag)
              EVDW=EVDW-c6vdw*
     .     damp(distn,a6,spr(l,icfrag),spr(m,jcfrag),iswitchvdw)/distn6
             enddo
            enddo
creplicas
           enddo
          enddo
         enddo
        enddo
c
        write(6,*) icont,'  C6 ij terms'
c
        ebind=eks-efrag
        ebindwdv=ebind+EVDW
        ebpr=ebind*evm
        ebpvdwpr=ebindwdv*evm
        evdwpri=ebpvdwpr-ebpr
        ra=r*au
c
        write(14,888) r,ra,ebpr,ebpvdwpr,evdwpri
888     format(1x,2f7.3,3x,2f16.6,3x,f16.6)
c
        if (ebind.lt.emin) then
         emin=ebind
         req=r
        endif
        if (ebindwdv.lt.emin6) then
         emin6=ebindwdv
         req6=r
        endif
        ninter=(nfrag*(nfrag-1))/2
        c6ave=c6ave/float(ninter)
c
c from a.u. 
c
       eks=eks*2.
       EVDW=EVDW*2.
c
        write(6,*) 'R(a.u.)  EKS(Ry)     EVDW(Ry)   averaged C6(a.u.) :'
        write(6,757) r,eks,EVDW,c6ave
        write(6,*) ' WF averaged sum of spreads and distances:' 
        write(6,767) sijave/float(icont),distave/float(icont),distmin
757     format(f6.3,f13.6,2e12.3)
767     format('  <si+sj> <dist> min.-dist(a.u.) : ',3f9.3)
c
        goto 999
1000    continue
c
        close(14)
c
        write(6,787) emin*evm,req,req*au
        write(6,788) emin6*evm,req6,req6*au
787     format(//1x,'Min. Ebind(meV)= ',f9.3,
     f             '  at Req(a.u. A)= ',2f7.3,/' no VdW correction')    
788     format(/1x,'Min. Ebind(meV)= ',f9.3,
     f             '  at Req(a.u. A)= ',2f7.3,/' C6 VdW correction')
c
        stop
        end
c
      SUBROUTINE DOUBLES(SI,SJ,RES)
c
C This routine gives an estimate of the C6 Van der Waals
C coefficient using the information relative to the 
C Wannier functions and solving the original 6-D
C integral :
C
C   const.* int int dr dr' (w(r)*w(r'))/(w(r)+w(r'))/|r-r'|**6
C
C by using spherical coordinates and thus reducing to a
C 2-D integral, which is evaluated by means of the Simpson method.   
C
      IMPLICIT REAL*8 (a-h,o-z)
      parameter(NPOINTS=51)
      dimension w(NPOINTS),x(NPOINTS),y(NPOINTS)
c
c defines cutoffs :
c
      xc=3.*(0.567+0.5*dlog(si))
      yc=3.*(0.567+0.5*dlog(sj))
c
      alf=2.
      beta=(si/sj)**1.5
      a=1./beta
      hi=xc/float(NPOINTS-1) 
      hj=yc/float(NPOINTS-1) 
c
      wsum=0.
      do i=1,NPOINTS
       x(i)=float(i-1)*hi
       if (i.eq.1.or.i.eq.NPOINTS) then 
         w(i)=hi/3.
       else if (mod(i,2).eq.0) then
         w(i)=hi*4./3.
       else
         w(i)=hi*2./3.
       endif
       wsum=wsum+w(i)
      enddo
      do j=1,NPOINTS
       y(j)=float(j-1)*hj
       if (j.eq.1.or.j.eq.NPOINTS) then 
         w(j)=hj/3.
       else if (mod(j,2).eq.0) then
         w(j)=hj*4./3.
       else
         w(j)=hj*2./3.
       endif
       wsum=wsum+w(j)
      enddo
c
      sum=0.
      do i=1,NPOINTS
       do j=1,NPOINTS
737     format(' j x w ',i5,2f15.6)
747     format(' x y F ',2f10.3,f15.6)
        sum=sum+w(i)*w(j)*EFFE(x(i),y(j),a)
       enddo
      enddo
c
      RES=sum
c
707   format(/1x,e16.9/)
727   format(/1x,3f15.6/)
c
      return
      end
C
      FUNCTION EFFE(x,y,a)
      IMPLICIT REAL*8 (a-h,o-z)
c
      EFFE=x*x*y*y*exp(-x)*exp(-y)
      EFFE=EFFE/(a*exp(-x)+exp(-y))
c
      return
      end
C**********************************************************************
      subroutine hexpbc(x1,y1,z1,x2,y2,z2,a1,a2,a3)
C**********************************************************************
      implicit real*8 (a-h,o-z)
      dimension a1(3),a2(3),a3(3)
c
      c1=a1(1)/2.
      c2=a2(2)/2.
      c3=a3(3)/2.
      y0=y1
      y2=dmod(y1,a2(2))
      y2=y2-a2(2)*dint(y2/c2)
      z2=z1
      dx=(y2-y0)/sqrt(3.)
      x2=dmod(x1-dx,a1(1))
      x2=x2-a1(1)*dint(x2/c1)
c     ==--------------------------------------------------------------==
      return
      end
C**********************************************************************
      subroutine mypbc(x1,y1,z1,x2,y2,z2,h,hm1)
C
C**********************************************************************
      implicit real*8 (a-h,o-z)
      dimension h(3,3),hm1(3,3),r(3),s(3)
c
      r(1)=x1
      r(2)=y1
      r(3)=z1
c
      do i=1,3
       s(i)=0.
       do l=1,3
        s(i)=s(i)+hm1(l,i)*r(l)
       enddo
      enddo
c
      s(1)=s(1)-nint(s(1))
      s(2)=s(2)-nint(s(2))
      s(3)=s(3)-nint(s(3))
c
      do i=1,3
       r(i)=0.
       do l=1,3
        r(i)=r(i)+h(l,i)*s(l)
       enddo
      enddo
c
      x2=r(1)
      y2=r(2)
      z2=r(3)
c     ==--------------------------------------------------------------==
      return
      end
C**********************************************************************
      subroutine effvol(ifrag,n,effv)
C**********************************************************************
      implicit real*8 (a-h,o-z)
      include 'max-dimension.h'
      dimension rmin(3),rmax(3),nr(3)
      dimension imin(3),imax(3)
c
      if (n.eq.1) then
       effv=1.
       goto 999
      endif
c
c mesh spacing :
c
      deltar=0.02
c
      do j=1,3
       rmin(j)=100000.
       rmax(j)=-100000.
      enddo
c
      do i=1,n
       do j=1,3
        if (rwfc(j,i,ifrag).lt.rmin(j)) then 
         rmin(j)=rwfc(j,i,ifrag)
         imin(j)=i
        endif
        if (rwfc(j,i,ifrag).gt.rmax(j)) then 
         rmax(j)=rwfc(j,i,ifrag)
         imax(j)=i
        endif
       enddo
      enddo
      do j=1,3
       rmax(j)=rmax(j)+spr(imax(j),ifrag)
       rmin(j)=rmin(j)-spr(imin(j),ifrag)
       nr(j)=abs(rmax(j)-rmin(j))/deltar+1
      enddo
c      write(6,707) (nr(j),j=1,3)
707   format(' num. of mesh points : ',3i10)
c
      vsum=0.
      veff=0.
      r1=rmin(1)
      do i1=1,nr(1)
       r2=rmin(2)
       do i2=1,nr(2)
        r3=rmin(3)
        do i3=1,nr(3)
         nsph=0
         do i=1,n
          dis1=r1-rwfc(1,i,ifrag)
          dis2=r2-rwfc(2,i,ifrag)
          dis3=r3-rwfc(3,i,ifrag)
          distn=sqrt(dis1*dis1+dis2*dis2+dis3*dis3)
          if (distn.lt.spr(i,ifrag)) then 
           nsph=nsph+1
          endif
         enddo
         if (nsph.gt.0) then 
          weig=1./float(nsph)
          vsum=vsum+1.
          veff=veff+weig
         endif
c
         r3=r3+deltar
        enddo
        r2=r2+deltar
       enddo
       r1=r1+deltar
      enddo
c
      effv=veff/vsum
999   continue
c
      return
      end
c     ==--------------------------------------------------------------==
        subroutine gaussj(a,n)
c     ==--------------------------------------------------------------==
        integer m,n
        real*8 a(n,n),b(n,n)
        integer i,icol,irow,j,k,l,ll,indxc(3),indxr(3),
     c          ipiv(3)
        real*8 big,dum,pivinv
c
        m=n        
        do i=1,3 
         do j=1,3 
          b(i,j)=0.0
         enddo
        enddo
        do i=1,3 
         b(i,i)=1.
        enddo
c
        do j=1,n
         ipiv(j)=0
        enddo
        do i=1,n
         big=0.
         do j=1,n
          if (ipiv(j).ne.1) then
           do k=1,n
            if (ipiv(k).eq.0) then
             if (abs(a(j,k)).ge.big) then
              big=abs(a(j,k))
              irow=j
              icol=k
             endif
            else if (ipiv(k).gt.1) then
             stop 'singular matrix in gaussj'
            endif
           enddo
          endif
         enddo
         ipiv(icol)=ipiv(icol)+1
         if (irow.ne.icol) then
          do l=1,n
           dum=a(irow,l)
           a(irow,l)=a(icol,l)
           a(icol,l)=dum
          enddo
          do l=1,m
           dum=b(irow,l)
           b(irow,l)=b(icol,l)
           b(icol,l)=dum
          enddo
         endif
         indxr(i)=irow
         indxc(i)=icol
         if (a(icol,icol).eq.0.) stop 'singular matrix in gaussj'
         pivinv=1./a(icol,icol)
         a(icol,icol)=1.
         do l=1,n
          a(icol,l)=a(icol,l)*pivinv
         enddo
         do l=1,m
          b(icol,l)=b(icol,l)*pivinv
         enddo
         do ll=1,n
          if (ll.ne.icol) then
           dum=a(ll,icol)
           a(ll,icol)=0.
           do l=1,n
            a(ll,l)=a(ll,l)-a(icol,l)*dum
           enddo
           do l=1,m
            b(ll,l)=b(ll,l)-b(icol,l)*dum
           enddo
          endif
         enddo
        enddo
        do l=n,1,-1
         if (indxr(l).ne.indxc(l)) then
          do k=1,n
           dum=a(k,indxr(l))
           a(k,indxr(l))=a(k,indxc(l))  
           a(k,indxc(l))=dum
          enddo
         endif
        enddo
        return
        end 
c
c     ==--------------------------------------------------------------==
        subroutine readwfc(izcenter)
c     ==--------------------------------------------------------------==
C
C   From want*.out (PW-WanT) format to WANNIER_CENTER (CPMD-like) format
C
C    format:
C
C             i   xwf_i(au)  ywf_i(au)  zwf_i(au)   spread_i(au**2)
C
      implicit REAL*8 (a-h,o-z)
      include 'max-dimension.h'
      CHARACTER*35 LINE
c
10    READ(10,717) LINE
      IF(LINE.EQ.'                in units of Bohr   ') THEN
       read(10,*)
       read(10,*)
       READ(10,'(28x,f8.4)') zpar
      ENDIF
c
      iread=0
      IF(LINE.EQ.'  =  Convergence Achieved          ') iread=1
      IF(LINE.EQ.'  =  Max number of iteration reache') iread=1
      IF(iread.eq.1) THEN
       read(10,*)
       read(10,*)
       read(10,*)
       read(10,*)
       read(10,*)
       read(10,*)
       read(10,*)
       read(10,*)
11     READ(10,'(11x,i3,5x,3f13.6,12x,f13.6)',ERR=100,END=100) i,x,y,z,s
c
       if (izcenter.eq.1) then
c
c centers are relocated between -zpar/2. and +zpar/2. :
c
        zpard2=zpar/2.
        zpartop=zpard2
        zparbot=-zpard2
        if (z.gt.zpartop) z=z-zpar
        if (z.lt.zparbot) z=z+zpar
       endif
c
       rwfc(1,i,1)=x
       rwfc(2,i,1)=y
       rwfc(3,i,1)=z
       spr(i,1)=sqrt(s)
c        
       goto 11
      ENDIF
c
      goto 10
707   format(i10,3x,3f12.3,5x,f12.3)
717   FORMAT(bz,a35)
100   continue
c
      end
c
c     ==--------------------------------------------------------------==
        subroutine readion
c     ==--------------------------------------------------------------==
c
      implicit REAL*8 (a-h,o-z)
      include 'max-dimension.h'
      CHARACTER*42 LINE
c
10    READ(11,717) LINE
      IF(LINE.EQ.'  Atomic positions: (cart. coord. in Bohr)') THEN
       i=1
11     READ(11,'(24x,3f12.7)',ERR=100,END=100) (rion(ic,i),ic=1,3)
       i=i+1
       goto 11
      ENDIF
c
      goto 10
c
100   continue
      if ((i-1).gt.nionx) then 
       write(6,*)
       write(6,*) ' nionx too small ! '
       write(6,*) ' increase nionx in max-dimension.h'
       stop
      endif
717   FORMAT(bz,a42)
c
      end
C     ==================================================================
      REAL*8 FUNCTION DAMP(R,a,s1,s2,iswitchvdw)
C     ==--------------------------------------------------------------==
C ***-1: f(R)=1/[1+exp(-a*(R/Rs-1))]     
      IMPLICIT NONE
      INTEGER iswitchvdw
      REAL*8 a,R,s1,s2
      REAL*8 ai,aj,ar,beta,cobt,sibt,es,Rs,soij,tau
      REAL*8 dlogso,root3
      root3=DSQRT(3.D0)
c
       IF (a.gt.1.D-5) THEN
c
        IF(iswitchvdw.eq.1) THEN
         Rs=root3*((0.769D0+0.5D0*DLOG(s1))*s1
     .            +(0.769D0+0.5D0*DLOG(s2))*s2)
        ELSEIF(iswitchvdw.eq.2.or.iswitchvdw.eq.3) THEN
         Rs=1.309D0*(s1+s2)
        ENDIF
        es=DEXP(-a*(R/Rs-1.D0))
        damp=1.D0/(1.D0+es)
c
       ELSE
c
c  Damping function after Mol. Phys. 107, 999 (2009):
c
        IF(iswitchvdw.eq.2.or.iswitchvdw.eq.3) THEN
         Rs=1.309D0*(s1+s2)
         es=DEXP(-a*(R/Rs-1.D0))
         damp=1.D0/(1.D0+es)
        ELSEIF(iswitchvdw.eq.1) THEN
         ai=root3/s1
         aj=root3/s2
         beta=0.5D0*(ai+aj)*R
         tau=(ai-aj)/(ai+aj)
         IF(DABS(tau).lt.1.D-05) THEN
          ar=ai*R
          soij=DEXP(-ar)*(1.D0+ar+ar*ar/3.)
         ELSE
          cobt=DCOSH(beta*tau)
          sibt=DSINH(beta*tau)
          soij=cobt*beta/tau+(1.D0+beta-1.D0/tau/tau)*sibt
          soij=soij*(ai*aj)**1.5D0*R**3/beta**4/tau*DEXP(-beta)
         ENDIF
         dlogso=DLOG(DABS(soij))
         damp=1.D0-soij*soij*(1.D0-2.D0*dlogso*(1.D0-dlogso))
        ENDIF

       ENDIF
c
       RETURN
       END
